%  integrate backwards to generate the switching curve for a standard
%  oscillator system with no zero.
%
% Change alpha from to 0.5 to demonstrate whole.


clear all;
clc;
addpath('functions')
saveon = o; % save figures?


fig1_name = 'brake01.pdf';
fig1_path = fullfile(getfigpath(), fig1_name); 
load('CS.mat', 'CS'); % Color definitions.

%============================================
% Model Definition
w    = 10;
bo   = 100;

c    = bo/w^2;
zeta = .15;
wd = w*sqrt(1-zeta^2);
%make the state space matrices
A    = [0   1;
      -(w)^2, -2*w*zeta;]; 
B    = [0; bo];
C    = [1 0];
D    = [0; 0];

b1 = B(1);
b2 = B(2);

params.A = A;
params.B = B;

trun = 2*pi/wd;
sys  = ss(A, B, C, 0);

% Define the control
% umax    = 1; %saturator limit
alpha = 1; %discount factor

x1r = -0.3*c;
ubp = 1-x1r/c;
ubm = -1-x1r/c;

dx = 0.0051;
x1p = (0:dx:2*alpha*c)';
x1n = (-2*alpha*c:dx:0-dx)';

dt = (pi/w)/1000;
step_sz = dt;
T0 = (0:dt:pi/w)';
TB = flipud(T0);
x0p = [0; 0];



RR = @(theta) [cos(theta), -sin(theta);
               sin(theta), cos(theta)];
% ============================
% Integrate backwards
params.u = 1*alpha*ubp;
[T, Sp]  = ode45(@(t,x)dyn(t,x,params), TB, x0p);

params.u = 1*alpha*ubm;
[T, Sn]  = ode45(@(t,x)dyn(t,x,params), TB, x0p);

kk = length(Sn(:,1));

k_s  = floor(linspace(80,kk, 10));
k_s  = [80, k_s, kk-80,];

xp_s = []; xn_s = [];
for i = 1:length(Sn)
    
% From the Scurve in negative half plane.
    params.u = ubp;
    x0p      = [Sn((i), 1), Sn((i), 2)]';
    
    [T, xp]  = ode45(@(t,x)dyn(t,x,params), TB, x0p);
    xp_new(i, :) = xp(end, :);
    
    % From the Scurve in positive half plane.
    params.u = ubm;
    x0n      = [Sp((i), 1), Sp((i), 2)]';
    [T, xn]  = ode45(@(t,x)dyn(t,x,params), TB, x0n);
    xn_new(i, :) = xn(end, :);
    
    
    
    if find(k_s ==i)
        xn_s     = [xn_s, xn];
        xp_s     = [xp_s, xp];
    end

end



% ----------------------------------------------------------------------- %
%                         Plot, format axes etc.                          %
% ----------------------------------------------------------------------- %


F1    = figure(1);clf  % Create Figure

axes('FontName','Times New Roman') % Set axis font style
box('on'); % Define box around whole figure
hold on;

width = 3.5;
height = 3;
set(F1, 'Units', 'Inches', 'Position', [0, 0, width, height],...
    'PaperUnits', 'Inches', 'PaperSize', [width, height])
set(F1, 'Color', 'w');

% ----------------------------------------------------------------------- %
%                              Plot stuff.                                %

% ------------
% plot s-curve

lw = 1.5;
plot(Sn(:,1), Sn(:,2),'--k', 'LineWidth', lw)
plot(Sp(:,1), Sp(:,2), 'k', 'LineWidth', lw)
grid on


%===================
% Plot backwards integrated.
sz = size(xp_s);
ks = sz(2);

kx1s = (1:2:ks(end));
kx1s(end-2:end-1)=[];
kx2s = (2:2:ks(end));
kx2s(end-2:end-1)=[];

% plot(xp_s(1:100:end,kx1s), xp_s(1:100:end,kx2s), '-.k')
plot(xp_s(1:10:end,kx1s), xp_s(1:10:end,kx2s), '-.k')
plot(xn_s(:,kx1s), xn_s(:,kx2s), ':k')


%===================
% Connect the dots, of the family of backwards integrations
plot(xp_new(:,1), xp_new(:,2), 'k', 'LineWidth', lw)
plot(xn_new(:,1), xn_new(:,2), '--k', 'LineWidth', lw)

% build the axes
clc
ax=gca;

xlim([1.2*min(min(xn_s(:, kx1s))),  1.2*max(max(xp_s(:, kx1s)))]);
xlm = xlim;
Nx1   = ceil(xlm(1)/c);
Nx2   = floor(xlm(2)/c);
nxdiff = 1;
xdiff = nxdiff*c;

XTICKS = [Nx1*xdiff/nxdiff:xdiff:Nx2*xdiff/nxdiff];

sig = zeta*w;
phi = sig*pi/wd;
chi = (1+exp(phi));
XTICKS = [chi*c*ubm, c*ubm, 0, c*ubp, chi*c*ubp];
set(ax, 'XTick', XTICKS);
set(ax, 'XTickLabel', []);


ylm = ylim;

Ny1    = ceil(ylm(1)/((bo)/w));
Ny2    = floor(ylm(2)/((bo)/w));
nydiff = 1;
ydiff  = nydiff*bo/w;

ylim([1.2*min(min(xp_s)), 1.4*max([max(xn_s(:,end))])])

set(ax, 'YTick', [0]);
set(ax, 'YTickLabel', [])


yticks=get(ax, 'ytick');
xticks=get(ax, 'xtick');
xlim([xn_new(end,1), xp_new(end,1)]);
ylim([-45 , 60]);

%==========
% Build strings for custom labels.

my_xticks = {'$\psi c\bar{u}^-$',...
             '$c\bar{u}^-$', '$0$', '$c\bar{u}^+$',...
             '$\psi c\bar{u}^+$'};

my_yticks = '$0$';

xlm = xlim;
ylm = ylim;

axs=axis;
HorizontalOffset=4.1*c;
VertOffset=1.2*1000;

xdist = xlm(2) - xlm(1);
ydist = ylm(2) - ylm(1);

if 1
    yax = xlm(1);
    xax = ylm(1);
    text(yax, yticks,my_yticks,'HorizontalAlignment','Right','interpreter', 'latex',...
          'FontSize', 12);
end

for i=1:1:length(my_xticks)
    if i==1
        s(i) = text(xticks(i)-.35*c, xax(1) + ydist*.0, my_xticks(i),'HorizontalAlignment','Center',...
          'VerticalAlignment', 'Top', 'interpreter', 'latex', 'FontSize',10);   
    else
      s(i) = text(xticks(i), xax(1) + ydist*.0, my_xticks(i),'HorizontalAlignment','Center',...
          'VerticalAlignment', 'Top', 'interpreter', 'latex', 'FontSize',10);   
    end

end

grid on


% Create xlabel
hxlab=xlabel('$x_{1e}$','FontSize',12,'FontName','Times New Roman','FontAngle','italic');

% Create ylabel
hylab=ylabel('$x_{2e}$','FontSize',12,'FontName','Times New Roman','FontAngle','italic','rot',90);
set(hylab, 'Units', 'Normalized', 'Position', [-0.065, .42, 0]);
set(hxlab, 'Units', 'Normalized', 'Position', [.5, -0.045, 0]);
set(hxlab, 'interpreter', 'latex');
set(hylab, 'interpreter', 'latex');



x1c1 = 2*c*ubp;
x1c2 = 2*c*abs(ubm);


x1c = ds2nfu([c*ubm, c*ubp], [0, 0]);
annotation('doublearrow', x1c, [.9, .9], 'headstyle', 'vback3');
text(x1c(1)+.02, .92, '$2cu^+$', 'interpreter', 'latex', 'units', 'normalized', 'FontSize', 12);

legend({'$\mathcal{S}^-$', '$\mathcal{S}^+$'}, 'interpreter', 'latex')



st(1) = text(0.1, 0.1, '$\mathcal{U}^+$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', 12);
st(2) = text(.9, .7, '$\mathcal{U}^-$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', 12);

st(3) = text(.645, .32, '$\mathcal{S}^+_1$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', 12);
st(4) = text(.86, .33, '$\mathcal{S}^+_2$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', 12);

st(5) = text(.45, .55, '$\mathcal{S}^-_1$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', 12);
st(6) = text(.16, .65, '$\mathcal{S}^-_2$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', 12);


stit = sprintf('$x_{1r} = %.2fcu_{max}$, $\\zeta = %.2f$', x1r/c, zeta);
title(stit, 'interpreter', 'latex', 'FontSize', 12);



if saveon
    export_fig(fig1_path, '-q101')
end
