%  Integrate backwards to generate the switching curve for a standard
%  oscillator system with no zero.
%

clear;
clc;
saveon = 0;

addpath('functions');
addpath('hatchfillpkg');

addpath('functions');
fig_name = 'brake02.pdf';
fig_path = fullfile(getfigpath(), fig_name); 



load('CS.mat', 'CS');

%============================================
% Model Definition

w    = 10;
bo   = 100;

c    = bo/w^2;
zeta = 0.3;
sigma = w*zeta;
wd = w*sqrt(1 - zeta^2);

psi = (1+exp(sigma*pi/wd));

%make the state space matrices
A    = [0,    1;
       -(w)^2, -2*sigma;]; 
B    = [0; bo];
C    = [1 0];
D    = [0; 0];

b1 = B(1);
b2 = B(2);

params.A = A;
params.B = B;

trun = 2*pi/w;
sys  = ss(A, B, C, 0);

% Define the control
umax    = 10; %saturator limit
alpha = 1; %discount factor

x1r = -0;
ubp = (1-x1r/c)*umax;
ubm = (-1-x1r/c)*umax;


dx = 0.0051;
x1p = (0:dx:2*alpha*c)';
x1n = (-2*alpha*c:dx:0-dx)';

% S1 = -w*sqrt(2*alpha*c*x1p - x1p.^2);
% S2 =  w*sqrt(-2*alpha*c*x1n - x1n.^2);

dt = (pi/wd)/1000;
step_sz = dt;
T0 = (0:dt:pi/wd)';
TB = flipud(T0);
x0p = [0; 0];



% ============================
% Integrate backwards
params.u = umax*alpha;
[T, Sp]  = ode45(@(t,x)dyn(t,x,params), TB, x0p);

params.u = -umax*alpha;
[T, Sn]  = ode45(@(t,x)dyn(t,x,params), TB, x0p);


[T1, Xbnd_neg] = ode45(@(t,x)dyn(t,x,params), TB, [psi*c*umax, 0]);
params.u = +umax;
[T1, Xbnd_pos] = ode45(@(t,x)dyn(t,x,params), TB, [-psi*c*umax, 0]);


kk = length(Sn(:,1));

k_s  = floor(linspace(80,kk, 10));
k_s  = [80, k_s, kk-80,];



% ----------------------------------------------------------------------- %
%                             Plot, format axes etc.                      %
% ----------------------------------------------------------------------- %

F1  = figure(1);clf 

fs1 = 10;
fs_lab = 12;
width = 3.;
height = 3;
set(F1, 'color', 'w',...
    'defaulttextinterpreter', 'latex',...
    'PaperUnits','inches',...
    'PaperSize',[width height],...
    'PaperPosition',[0,0,width,height],...
    'Units','inches',...
    'Position',[1,1,width,height]);


ht = .815;
ax1 = axes('Position', [.13,.12,1-.135, ht]);
hold on;
box('on');

% ==================
% plot s-curve

lw = 1.5;
% Do the shaded plot first, so the rest of curves go on top!

k_negbnd = find(Xbnd_neg(:,1)<=-psi*c*umax, 1, 'first');
k_posbnd = find(Xbnd_pos(:,1)>=psi*c*umax, 1, 'first');
x1_shaded = linspace(-psi*c*umax, psi*c*umax, k_posbnd);
x2_shaded = interp1(Xbnd_neg(1:k_negbnd,1), Xbnd_neg(1:k_negbnd,2), x1_shaded);

% 'k', .15
plotshaded(x1_shaded, [0*x2_shaded; x2_shaded],'k', .15);
plotshaded(-x1_shaded, [0*x2_shaded; -x2_shaded],'k', .15);

% Now, plot the dashed outline of the region of interest. 
plot(Xbnd_neg(1:k_negbnd+1,1), Xbnd_neg(1:k_negbnd+1,2), ':k');    % top
plot(Xbnd_pos(1:k_posbnd,1), Xbnd_pos(1:k_posbnd,2), ':k')         % bottom
plot([psi*c*umax, psi*c*umax], [Xbnd_pos(k_posbnd,2), 0], ':k');   % right
plot([-psi*c*umax, -psi*c*umax], [Xbnd_neg(k_negbnd,2), 0], ':k')  % left

ymin = min(Xbnd_pos(1:k_posbnd,2));
ymax = max(Xbnd_neg(1:k_negbnd+1,2));
ylim([ymin, ymax])

% Now, plot the switching curve
h_sn = plot(Sn(:,1), Sn(:,2),'--k', 'LineWidth', lw);
h_sp = plot(Sp(:,1), Sp(:,2), 'k', 'LineWidth', lw);


grid on;


%%%%%% build the axes
ax=gca;

xlim([1.2*min(min(Sn(:,1))),  1.2*max(max(Sp(:,1)))]);
xlm = xlim;
Nx1   = ceil(xlm(1)/c/umax);
Nx2   = floor(xlm(2)/c/umax);
nxdiff = 1;
xdiff = nxdiff*c*umax;


% [Nx1*xdiff/nxdiff:xdiff:Nx2*xdiff/nxdiff]
XTICKS = [psi*c*ubm, c*ubm, 0, c*ubp, psi*c*ubp];
set(ax, 'XTick', XTICKS);
set(ax, 'XTickLabel', [])


ylm = ylim;

Ny1    = ceil(ylm(1)/((bo)*umax/w));
Ny2    = floor(ylm(2)/((bo)*umax/w));
nydiff = 1;
ydiff  = nydiff*bo*umax/w;

% ylim([1.2*min(min(Sp(:,2))), 1.4*max([max(Sn(:,2))])])
set(ax, 'YTick', [0]);
set(ax, 'YTickLabel', ['0'])


yticks=get(ax, 'ytick');
xticks=get(ax, 'xtick');

%==========
% Build strings for custom labels.
my_xticks = {'$\psi c\bar{u}^-$',...
             '$c\bar{u}^-$', '$0$', '$c\bar{u}^+$',...
             '$\psi c\bar{u}^+$'};

xlm = xlim;
ylm = ylim;

plot([xlm(1), -psi*c*umax], [0, 0], '--k', 'LineWidth', lw)
plot([psi*c*umax, xlm(2)], [0, 0], 'k', 'LineWidth', lw)

axs=axis;
HorizontalOffset=4.1*c;
VertOffset=1.5*1000;

xdist = xlm(2) - xlm(1);
ydist = ylm(2) - ylm(1);

yax = xlm(1);
xax = ylm(1);


for i=1:length(my_xticks)
    if i==1
        s(i) = text(xticks(i)-.15*c, xax(1) - ydist*.01, my_xticks(i),'HorizontalAlignment','Center',...
          'VerticalAlignment', 'Top', 'interpreter', 'latex', 'FontSize',fs1);   
    elseif i==4
      s(i) = text(xticks(i)+.2*c*umax, xax(1) - ydist*.01, my_xticks(i),'HorizontalAlignment','Center',...
          'VerticalAlignment', 'Top', 'interpreter', 'latex', 'FontSize',fs1);   
    else
      s(i) = text(xticks(i), xax(1) - ydist*.01, my_xticks(i),'HorizontalAlignment','Center',...
      'VerticalAlignment', 'Top', 'interpreter', 'latex', 'FontSize',fs1);   
    end

end

grid on

% Create xlabel
hxlab=xlabel('$x_{1e}$','FontSize',fs_lab,'FontName','Times New Roman','FontAngle','italic');

% Create ylabel
hylab=ylabel('$x_{2e}$','FontSize',fs_lab,'FontName','Times New Roman','FontAngle','italic','rot',90);
set(hylab, 'Units', 'Normalized', 'Position', [-.07, 0.5, 0]);
set(hxlab, 'Units', 'Normalized', 'Position', [.5, -0.055, 0]);
set(hxlab, 'interpreter', 'latex');
set(hylab, 'interpreter', 'latex');



[x1fp, x2f] = ds2nfu([c*ubm, c*ubp], [0,0]);
annotation('doublearrow', x1fp, x2f, 'headstyle', 'vback3');

[x1fp, x2f] = ds2nfu([c*ubm, 0.3*c*ubm], [-400,0]);
annotation('textarrow', x1fp, x2f, 'headstyle', 'vback3',...
        'String', '$\mathcal{H}_{eq}$', 'interpreter', 'latex', 'FontSize', 12);

hleg = legend([h_sn, h_sp], {'$f_{to}^-$', '$f_{to}^+$'}, 'interpreter', 'latex', 'Box', 'off');
hleg.Position = [0.5375 0.7514 0.5323 0.1793];
hleg.FontSize = fs1;


st(1) = text(0.1, 0.1, '$\mathcal{U}^+$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', fs1);
st(2) = text(.9, .7, '$\mathcal{U}^-$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', fs1);

st(3) = text(.66, .33, '$f_{to}^+$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', fs1);
st(4) = text(.28, .67, '$f_{to}^-$', 'units', 'normalized', 'interpreter', 'latex', 'FontSize', fs1);


box('on'); % Define box around whole figure


if saveon
    print(gcf, '-dpdf', 'latex/figures/brake02.pdf', '-r600', '-opengl'); 
end

